
#include <bootkeys.h>
#include <Setup.h>
#include <CString.h>
#include <string.h>
#include <mem.h>
	
static const char *StrKeys = "Additional keys";		
		
CBootKeys::CBootKeys(CSetup &SetupToUse, CBootItems &BootItemsToUse, CKeyboard &KeyboardToUse):
	Setup(SetupToUse),
	BootItems(BootItemsToUse),
	Keyboard(KeyboardToUse)
{
}

CBootKeys::~CBootKeys()
{
}

void CBootKeys::CreateControls()
{
	KeysGroupBevel = new CBevel(BEVEL_FRAME,true,28,237,337,122,false);
	KeysGroupLabel = new CLabel(StrKeys,STYLE_REGULAR,false,17,36,230,false);
	KeysEdit = new CEdit("",48,false,false,52,270,289,false,this);
	KeyName = new CLabel("",STYLE_REGULAR,true,17,252,266,false);
	BackspaceBtn = new CButton("Backspc",53,298,67,25,false,this);
	TabBtn = new CButton("Tab",126,298,67,25,false,this);
	ShiftTabBtn = new CButton("Shift-Tab",199,298,67,25,false,this);
	EscapeBtn = new CButton("Escape",272,298,67,25,false,this);
}

void CBootKeys::InitializeControls(CTabControl *TabControl)
{
	TabControl->AddControl(2,KeysGroupBevel);
	TabControl->AddControl(2,KeysGroupLabel);
	TabControl->AddControl(2,KeysEdit);
	TabControl->AddControl(2,KeyName);
	TabControl->AddControl(2,BackspaceBtn);
	TabControl->AddControl(2,TabBtn);
	TabControl->AddControl(2,ShiftTabBtn);
	TabControl->AddControl(2,EscapeBtn);
}

void CBootKeys::ConnectEventHandlers()
{
	KeysEdit->OnKeyPress((TWndOnKeyPress)KeysEditKeyPress);
	BackspaceBtn->OnClick((TWndOnClick)BackspaceBtnClick);
	TabBtn->OnClick((TWndOnClick)TabBtnClick);
	ShiftTabBtn->OnClick((TWndOnClick)ShiftTabBtnClick);
	EscapeBtn->OnClick((TWndOnClick)EscapeBtnClick);
}

void CBootKeys::DisableControls()
{
	KeysEdit->Disable();
	BackspaceBtn->Disable();
	TabBtn->Disable();
	ShiftTabBtn->Disable();
	EscapeBtn->Disable();
}

void CBootKeys::EnableControls()
{
	KeysEdit->Enable();
	BackspaceBtn->Enable();
	TabBtn->Enable();
	ShiftTabBtn->Enable();
	EscapeBtn->Enable();
}

void CBootKeys::InstallControls(CForm *Form)
{
	Form->AddControl(KeysGroupBevel);
	Form->AddControl(KeysGroupLabel);
	Form->AddControl(KeysEdit);
	Form->AddControl(KeyName);
	Form->AddControl(BackspaceBtn);
	Form->AddControl(TabBtn);
	Form->AddControl(ShiftTabBtn);
	Form->AddControl(EscapeBtn);
}

void CBootKeys::RealignText()
{
	bool Visible;

	Visible = KeysGroupLabel->IsVisible();
	KeysGroupLabel->SetVisible(false);
	KeysGroupLabel->SetCaption(StrKeys);
	KeysGroupLabel->SetVisible(Visible);
}


//--------------------
void CBootKeys::KeysEditKeyPress(CBootKeys &BootKeys, int &Key)
{
	switch (Key) {
		case KEY_ESCAPE:
			memset(BootKeys.BootItems.Get(BootKeys.Setup.BootItemIndex)->Keys,0,32);
			BootKeys.UpdateKeyDisplay();
			break;
		case KEY_BACKSPACE:
			BootKeys.RemoveKey();
			Key = KEY_END;
			break;
		default:
			BootKeys.AddKey(Key);
			Key = KEY_END;
			break;
	}
	BootKeys.Setup.SetIgnoreNextKey();
}

void CBootKeys::BackspaceBtnClick(CBootKeys &BootKeys)
{
	BootKeys.AddKey(0x0e08);
}

void CBootKeys::TabBtnClick(CBootKeys &BootKeys)
{
	BootKeys.AddKey(0x0f09);
}

void CBootKeys::ShiftTabBtnClick(CBootKeys &BootKeys)
{
	BootKeys.AddKey(0x0f00);
}

void CBootKeys::EscapeBtnClick(CBootKeys &BootKeys)
{
	BootKeys.AddKey(0x011b);
}


void CBootKeys::AddKey(int Key)
{
	unsigned short *KeyList = BootItems.Get(Setup.BootItemIndex)->Keys;
	int Index;

	for (Index = 0; KeyList[Index]; ++Index);
	if (Index < 15) {
		KeyList[Index] = Key;
		UpdateKeyDisplay();
	}
}

void CBootKeys::RemoveKey()
{
	unsigned short *KeyList = BootItems.Get(Setup.BootItemIndex)->Keys;
	int Index;

	for (Index = 0; KeyList[Index]; ++Index);
	if (Index) {
		KeyList[Index - 1] = 0;
		UpdateKeyDisplay();
	}
}

void CBootKeys::GetKeysString(int ItemIndex, CString &KeysString)
{
	unsigned short *KeyList = BootItems.Get(ItemIndex)->Keys;
	char ShortKeyName[32];
	char FullKeyName[32];
	int Index;

	for (Index = 0; KeyList[Index]; ++Index) {
		if (Index) {
			KeysString += ".";
		}
		GetKeyName(KeyList[Index],FullKeyName,ShortKeyName);
		KeysString += ShortKeyName;
	}
}

void CBootKeys::UpdateKeyDisplay()
{
	CString KeysStr;

	GetKeysString(Setup.BootItemIndex,KeysStr);
	KeysEdit->SetText(KeysStr);
	Setup.DisplayBootItem(Setup.BootItemIndex,true);
}

void CBootKeys::GetKeyName(int KeyCode, char *FullKeyName, char *ShortKeyName)
{
	bool Shift, Ctrl, Alt;

	Keyboard.IsFuncKey(KeyCode,Shift,Ctrl,Alt);
	Keyboard.GetKeyName(KeyCode,FullKeyName);
	Keyboard.GetKeyStatus(Ctrl,Alt);
	if (!IsSpecialKey(KeyCode,ShortKeyName)) {
		if (Shift) {
			ShortKeyName[0] = 'S';
			strcpy(&ShortKeyName[1],&FullKeyName[6]); // FullKeyName - "Shift-"
		}
		else if (Ctrl) {
			ShortKeyName[0] = 'C';
			strcpy(&ShortKeyName[1],&FullKeyName[5]); // FullKeyName - "Ctrl-"
		}
		else if (Alt) {
			ShortKeyName[0] = 'A';
			strcpy(&ShortKeyName[1],&FullKeyName[4]); // FullKeyName - "Alt-"
		}
		else {
			strcpy(ShortKeyName,FullKeyName);
		}
	}
}

bool CBootKeys::IsSpecialKey(int KeyCode, char *ShortKeyName)
{
	int Index;

	for (Index = 0; Index < 27; ++Index) {
		if (SpecialKeyList[Index].KeyCode == KeyCode) {
			strcpy(ShortKeyName,SpecialKeyList[Index].KeyName);
			return true;
		}
	}
	return false;
}

CBootKeys::CSpecialKey CBootKeys::SpecialKeyList[27] = {
	0x011b,"esc", 0x0e08,"bs", 0x0e7f,"Cbs", 0x0f09,"tab", 0x0f00,"Stab", 0x1c0d,"ret", 0x1c0a,"Cret",
	0x4700,"hm", 0x7700,"Chm", 0x4800,"cu", 0x4900,"pu", 0x8400,"Cpu", 0x4b00,"cl", 0x7300,"Ccl",
	0x4d00,"cr", 0x7400,"Ccr", 0x4f00,"end", 0x7500,"Cend", 0x5000,"cd", 0x5100,"pd", 0x7600,"Cpd",
	0x5200,"ins", 0x5300,"del", 0x5f80,"slp", 0x7200,"Csreq", 0x3920, " ", 0x4e2b, "k+"
};


