
#include <MouseSet.h>

	 
static const char *StrMoTypeLabel = "Mouse type";
static const char *StrMoSpeedLabel = "Mouse speed";
static const char *StrMoSamplingLabel = "PS/2 sampling rate";
	 
	 
CMouseSettings::CMouseSettings(CXOSLData &XoslDataToUse, CDialogs &DialogsToUse, CMouse &MouseToUse):
	XoslData(XoslDataToUse), Dialogs(DialogsToUse), Mouse(MouseToUse)
{
	Initialized = false;
}

CMouseSettings::~CMouseSettings()
{

}

void CMouseSettings::CreateControls()
{
	TypeGroup = new CBevel(BEVEL_FRAME,TRUE,24,32,161,217,FALSE);
	TypeLabel = new CLabel(StrMoTypeLabel,STYLE_REGULAR,FALSE,17,32,25,FALSE);
	MouseGroup = new CRadioGroup(56,48,120,144,FALSE,this);
	ApplyBtn = new CButton("Apply",67,204,75,25,FALSE,this);

	SpeedGroup = new CBevel(BEVEL_FRAME,TRUE,200,32,241,41,FALSE);
	SpeedLabel = new CLabel(StrMoSpeedLabel,STYLE_REGULAR,FALSE,17,208,25,FALSE);
	SpeedSlider = new CTrackBar(0,8,7,228,48,201,FALSE,this);
	SamplingGroup = new CBevel(BEVEL_FRAME,TRUE,200,96,241,41,FALSE);
	SamplingLabel = new CLabel(StrMoSamplingLabel,STYLE_REGULAR,FALSE,17,208,89,FALSE);
	SamplingSlider = new CTrackBar(0,6,0,228,112,201,FALSE,this);
}

void CMouseSettings::InitializeControls(CTabControl *TabControl)
{
	TabControl->AddControl(2,TypeGroup);
	TabControl->AddControl(2,TypeLabel);
	TabControl->AddControl(2,MouseGroup);
	TabControl->AddControl(2,SpeedGroup);
	TabControl->AddControl(2,SpeedLabel);
	TabControl->AddControl(2,SpeedSlider);
	TabControl->AddControl(2,SamplingGroup);
	TabControl->AddControl(2,SamplingLabel);
	TabControl->AddControl(2,SamplingSlider);
	TabControl->AddControl(2,ApplyBtn);

	MouseGroup->AddButton("Serial COM1");
	MouseGroup->AddButton("Serial COM2");
	MouseGroup->AddButton("Serial COM3");
	MouseGroup->AddButton("Serial COM4");
	MouseGroup->AddButton("PS/2");
	MouseGroup->AddButton("None");

}

void CMouseSettings::ConnectEventHandlers()
{
	SpeedSlider->OnChange((TTrackBarChange)SpeedChange);
	SamplingSlider->OnChange((TTrackBarChange)SamplingChange);
	ApplyBtn->OnClick((TWndOnClick)ApplyClick);

}

void CMouseSettings::InitializeData()
{
	TMouseData *MouseData;

	MouseData = XoslData.GetMouse();

	MouseGroup->SetChecked(MouseData->MouseType);
	SpeedSlider->SetValue(MouseData->MouseSpeed);
	SamplingSlider->SetValue(MouseData->PS2Sampling);
	SpeedChange(*this,MouseData->MouseSpeed);
	SamplingChange(*this,MouseData->PS2Sampling);
	
	Initialized = true;
}

void CMouseSettings::InstallControls(CForm *Form)
{
	PrefForm = Form;
	Form->AddControl(TypeGroup);
	Form->AddControl(TypeLabel);
	Form->AddControl(MouseGroup);
	Form->AddControl(ApplyBtn);
	Form->AddControl(SpeedGroup);
	Form->AddControl(SpeedLabel);
	Form->AddControl(SpeedSlider);
	Form->AddControl(SamplingGroup);
	Form->AddControl(SamplingLabel);
	Form->AddControl(SamplingSlider);
}

void CMouseSettings::RealignText()
{
	bool Visible;

	Visible = TypeLabel->IsVisible();
	
	TypeLabel->SetVisible(false);
	SpeedLabel->SetVisible(false);
	SamplingLabel->SetVisible(false);
	
	TypeLabel->SetCaption(StrMoTypeLabel);
	SpeedLabel->SetCaption(StrMoSpeedLabel);
	SamplingLabel->SetCaption(StrMoSamplingLabel);
	
	TypeLabel->SetVisible(Visible);
	SpeedLabel->SetVisible(Visible);
	SamplingLabel->SetVisible(Visible);
}

// --------------------- event handlers --------------------- 

void CMouseSettings::SpeedChange(CMouseSettings &MouseSettings, int Value)
{
	if (MouseSettings.Initialized) {
		MouseSettings.Mouse.SetSpeed(Value);
		MouseSettings.XoslData.GetMouse()->MouseSpeed = Value;
	}
}

void CMouseSettings::SamplingChange(CMouseSettings &MouseSettings, int Value)
{
	if (MouseSettings.Initialized) {
		MouseSettings.Mouse.SetSampling(Value);
		MouseSettings.XoslData.GetMouse()->PS2Sampling = Value;
	}
}

void CMouseSettings::ApplyClick(CMouseSettings &MouseSettings)
{
	int LastPort;
	int MousePort;
	int Checked;
	CString Message;
	static const char *MsgBoxTitle = "Mouse";
	static const int PortList[] = {
		MOUSE_COM1,MOUSE_COM2,MOUSE_COM3,MOUSE_COM4,MOUSE_PS2
	};

	Graph->HideCursor();
	LastPort = MouseSettings.Mouse.GetMouse();
	Checked = MouseSettings.MouseGroup->GetChecked();
	if (Checked == 5) {
		MouseSettings.Mouse.SetMouse(-1);
		MouseSettings.XoslData.GetMouse()->MouseType = -1;
	}
	else {
		MousePort = PortList[Checked];
		Message = "Initializing ";
		Message += MouseSettings.Mouse.MousePortName(MousePort);
		Message += "...";
		MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
		if (MouseSettings.Mouse.SetMouse(MousePort) == -1) {
			MouseSettings.Mouse.SetMouse(LastPort);
			Message += "failed!";
			MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
			if (LastPort != -1)
				Graph->ShowCursor();
		}
		else {
			MouseSettings.XoslData.GetMouse()->MouseType = MousePort;
			Message += "done";
			MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
			Graph->ShowCursor();
		}
		SpeedChange(MouseSettings,MouseSettings.SpeedSlider->GetValue());
		SamplingChange(MouseSettings,MouseSettings.SamplingSlider->GetValue());
	}
}







